<?php
namespace Delo;

use \Registry;
use \Language;
use \Session;
use \Config;
use \Loader;
use \ModelSettingSetting;
use \ModelLocalisationOrderStatus;

class DeloSettings
{
    const PREFIX_SETTINGS_VERSION_2 = 'delo_';
    const PREFIX_SETTINGS_VERSION_3 = 'payment_delo_';

    /** @var DeloErrors */
    public $errors;
    /** @var Language */
    private $language;
    /** @var Session */
    private $session;
    /** @var Config */
    private $config;
    /** @var DeloLog */
    private $log;
    /** @var ModelSettingSetting */
    private $modelSetting;
    /** @var ModelLocalisationOrderStatus */
    private $modelOrderStatus = null;
    /** @var int версия CMS */
    private $version;
    /** @var string */
    public $prefixSettings = '';

    /**
     * @param Registry $registry
     * @throws \Exception
     */
    public function __construct($registry)
    {
        $this->session = $registry->get('session');
        $this->config = $registry->get('config');
        $this->errors = $registry->get('DeloErrors');
        $this->language = $registry->get('language');
        $this->log = $registry->get('DeloLog');
        $requirements = $registry->get('DeloRequirements');
        /** @var Loader $loader */
        $loader = $registry->get('load');
        $loader->model('setting/setting');
        $this->modelSetting = new ModelSettingSetting($registry);
        $this->version = $requirements->getMajorVersionCMS();
        try {
            $loader->model('localisation/order_status');
        } catch (\Exception $e) {
            /**
             * В OpenCart 2 нету localisation/order_status на фронте
             * перехватываем исключение, но ничего не делаем
             */
        }
        if (class_exists('\ModelLocalisationOrderStatus')) {
            $this->modelOrderStatus = new ModelLocalisationOrderStatus($registry);
        }
        $this->prefixSettings = $this->determinePrefixSettings();
    }

    /**
     * Возможные варианты языка
     * @return array
     */
    public function optionsLang()
    {
        return [
            'RU' => 'RU',
            'EN' => 'EN',
        ];
    }

    /**
     * Возможный вариант валюты
     * @return array
     */
    public function optionsCurrency()
    {
        return ['643' => '643'];
    }

    /**
     * Defaults значения настроек
     * @return array
     */
    public function defaultValues()
    {
        $defaultSettings = [
            'back_url_s'      => HTTPS_CATALOG . '?route=checkout/success',
            'back_url_f'      => HTTPS_CATALOG . '?route=checkout/failure',
            'url'             => isset($_SERVER['DELOBANK_ENVIRONMENT']) ? 'https://testmpi.myskb.ru/payment/start.wsm' : 'https://mpi.skbbank.ru/payment/start.wsm',
            'lang'            => 'RU',
            'currency'        => '643',
            'status'          => 0,
            'info'            => $this->language->get('info_default'),
            'payment_page'    => 0,
            'title'           => $this->language->get('title_default'),
            'terms'           => $this->language->get('terms_default'),
            'check_signature' => 1,
            'debug_mode'      => 0,
        ];
        // Дополняем те у котрых нет default значении, пустыми значениями, чтобы в БД заранее были записи для каждой настройки
        $fields = $this->settingFields();
        foreach ($fields as $fieldName => $fieldSetting) {
            if (!isset($defaultSettings[$fieldName])) {
                $defaultSettings[$fieldName] = '';
            }
        }
        return $defaultSettings;
    }

    /**
     * Все возможные поля настройки
     * @return array
     */
    public function settingFields()
    {
        return [
            'status' => [
                'filter' => [
                    'int',
                ],
                'validate' => [
                    'enable',
                ]
            ],
            'merch_id' => [
                'filter' => [
                    'default',
                ],
                'validate' => [
                    'required',
                    'pcid',
                    'pcidLength',
                ]
            ],
            'purchase_account_amount_id' => [
                'filter' => [
                    'default',
                ],
                'validate' => [
                    'required',
                    'pcid',
                    'pcidLength',
                ]
            ],
            'back_url_s' => [
                'filter' => [
                    'default',
                ],
                'validate' => [
                    'required',
                    'https',
                    'url',
                ]
            ],
            'back_url_f' => [
                'filter' => [
                    'default',
                ],
                'validate' => [
                    'required',
                    'https',
                    'url',
                ]
            ],
            'url' => [
                'filter' => [
                    'default',
                ],
                'validate' => [
                    'required',
                    'https',
                    'urlDelo',
                ]
            ],
            'lang' => [
                'filter' => [
                    'default',
                ],
                'validate' => [
                    'required',
                    'lang',
                ]
            ],
            'currency' => [
                'filter' => [
                    'default',
                ],
                'validate' => [
                    'required',
                    'int',
                    'currency',
                ]
            ],
            'order_status_before' => [
                'filter' => [
                    'int',
                ],
                'validate' => [
                    'status',
                ]
            ],
            'order_status_after' => [
                'filter' => [
                    'int',
                ],
                'validate' => [
                    'status',
                ]
            ],
            'order_status_failed' => [
                'filter' => [
                    'int',
                ],
                'validate' => [
                    'status',
                ]
            ],
            'title' => [
                'filter' => [
                    'default',
                ],
                'validate' => [
                    'required',
                ]
            ],
            'terms' => [
                'filter' => [
                    'default',
                ],
                'validate' => [
                ]
            ],
            'info' => [
                'filter' => [
                    'default',
                    // 'htmlspecialchars',
                ],
                'validate' => [
                ]
            ],
            'payment_page' => [
                'filter' => [
                    'int',
                ],
                'validate' => [
                ]
            ],
            'check_signature' => [
                'filter' => [
                    'int',
                ],
                'validate' => [
                    'enable',
                ]
            ],
            'debug_mode' => [
                'filter' => [
                    'int',
                ],
                'validate' => [
                    'enable',
                ]
            ],
        ];
    }

    /**
     * @param string $str
     * @param string $search
     * @return string
     */
    public function setRealDomain($str, $search = 'https://example-shop.ru/')
    {
        return str_replace($search, HTTPS_CATALOG, $str);
    }

    /**
     * @param array $data
     * @return array
     */
    public function filter($data)
    {
        $fields = $this->settingFields();
        $settings = $this->forEmptyValueFields();
        foreach ($fields as $fieldName => $fieldSetting) {
            if (isset($data[$this->prefixSettings . $fieldName])) {
                $val = $data[$this->prefixSettings . $fieldName];
                foreach ($fieldSetting['filter'] as $filter) {
                    $val = $this->{$filter . 'Filter'}($val);
                }
                $settings[$fieldName] = $val;
            }
        }
        return $settings;
    }

    /**
     * Это для checkbox-ов, когда они отключены то их значение не приходит, а это равнозначно отключенному состоянию = 0
     * @return array
     */
    private function forEmptyValueFields()
    {
        return [
            'status'          => 0,
            'payment_page'    => 0,
            'check_signature' => 0,
            'debug_mode'      => 0,
        ];
    }

    /**
     * @param string $val
     * @return int
     */
    private function intFilter($val)
    {
        return (int)$val;
    }

    /**
     * @param string $val
     * @return string
     */
    private function defaultFilter($val)
    {
        return trim($val);
    }

    /**
     * @param string $val
     * @return string
     */
    private function htmlspecialcharsFilter($val)
    {
        return htmlspecialchars($val, ENT_QUOTES | ENT_HTML5);
    }

    /**
     * Проверяет поля, если все хорошо true, иначе false
     * @param array $data
     * @return bool
     */
    public function validate($data)
    {
        $fields = $this->settingFields();
        $ok = true;
        foreach ($fields as $field => $fieldSetting) {
            if (isset($data[$field])) {
                foreach ($fieldSetting['validate'] as $validate) {
                    $error = $this->{$validate . 'Validate'}($data[$field]);
                    if (!empty($error)) {
                        $ok = false;
                        $this->errors->setError($error, $field);
                        break 1;
                    }
                }
            }
        }
        return $ok;
    }

    /**
     * Проверка, что поле обязательно задано, если не задано возвращает ошибку, иначе null
     * @param string $val
     * @return string|null
     */
    private function requiredValidate($val)
    {
        $error = null;
        if (empty($val)) {
            $error = $this->language->get('required_error');
        }
        return $error;
    }

    /**
     * Проверка, что путь до файла существует
     * Основан на проверке существования файла
     * @param string $path путь до файла
     * @return string|null
     */
    private function certValidate($path)
    {
        if (empty($path)) {
            return null;
        }
        $realPath = realpath(DIR_APPLICATION . '/../' . $path);
        if (!is_readable($realPath)) {
            return $this->language->get('cert_error_text');
        }
        return null;
    }

    /**
     * Ищет запрещенные символы в PCID
     * @param string $val
     * @return string
     */
    public function pcidValidate($val)
    {
        $error = null;
        if (preg_match('/[^a-z0-9]/i', $val)) {
            $error = $this->language->get('pcid_error_validate_text');
        }
        return $error;
    }

    /**
     * Проверка длины PCID
     * @param string $val
     * @return string
     */
    public function pcidLengthValidate($val)
    {
        $error = null;
        if (mb_strlen($val) != 32) {
            $error = $this->language->get('pcid_error_validate_length');
        }
        return $error;
    }

    /**
     * Проверяет есть ли язык среди возможных
     * @param string $val
     * @return string|null
     */
    private function langValidate($val)
    {
        $error = null;
        $langs = $this->optionsLang();
        if (!isset($langs[$val])) {
            $error = 'Язык "' . $val . '" не существует';
        }
        return $error;
    }

    /**
     * Проверяет есть ли валюта среди возможных
     * @param int $val
     * @return string|null
     */
    private function currencyValidate($val)
    {
        $error = null;
        $currencyArr = $this->optionsCurrency();
        if (!isset($currencyArr[$val])) {
            $error = $this->language->get('currency_ot exist_error');
        }
        return $error;
    }

    /**
     * Проверяет что в значении только числа
     * @param int $val
     * @return string|null
     */
    private function intValidate($val)
    {
        $error = null;
        $intVal = (string)(int)$val;
        if ($intVal != $val) {
            $error = $this->language->get('int_error');
        }
        return $error;
    }

    /**
     * Проверяет что это url
     * @param string $val
     * @return string|null
     */
    private function urlDeloValidate($val)
    {
        $error = null;
        $need = [
            'https://mpi.skbbank.ru/payment/start.wsm' => 1,
            'https://testmpi.myskb.ru/payment/start.wsm' => 1,
        ];
        if (!isset($need[$val])) {
            $error = $this->language->get('url_delo_error');
        }
        return $error;
    }

    /**
     * Проверяет что это url
     * @param string $url
     * @return string|null
     */
    private function urlValidate($url)
    {
        $error = null;
        if (!filter_var($url, FILTER_VALIDATE_URL)) {
            $error = $this->language->get('url_our_error');
        }
        return $error;
    }

    /**
     * Проверяет что это url
     * @param string $val
     * @return string|null
     */
    private function httpsValidate($val)
    {
        $error = null;
        if (strpos($val, 'https') === false) {
            $error = $this->language->get('https_error');
        }
        return $error;
    }

    /**
     * Вкл/выкл способа оплаты
     * @param int $val
     * @return string|null
     */
    private function enableValidate($val)
    {
        $error = null;
        $need = [
            0 => 'выключено',
            1 => 'включено',
        ];
        if (!isset($need[$val])) {
            $error = $this->language->get('enable_error');
        }
        return $error;
    }

    /**
     * Валидация статусов заказов
     * @param int $statusId
     * @return string|null
     */
    private function statusValidate($statusId)
    {
        if (!empty($this->modelOrderStatus)) {
            $orderStatuses = $this->modelOrderStatus->getOrderStatuses();
            $newOrderStatus = [];
            foreach ($orderStatuses as $status) {
                $newOrderStatus[$status['order_status_id']] = $status['name'];
            }
            $orderStatuses = $newOrderStatus;
        } else {
            $orderStatuses = [];
        }
        $error = null;
        if (!isset($orderStatuses[$statusId])) {
            $error = $this->language->get('status_error');
        }
        return $error;
    }

    /**
     * Выставляет ошибки в сессию
     */
    public function setErrorsInSession()
    {
        $this->errors->setErrorsInSession();
    }

    /**
     * Возвращает ошибки формы из сессии если они там есть
     * @return array
     */
    public function getErrorsFromSession()
    {
        return $this->errors->getErrorsFromSession();
    }

    /**
     * Получение значении настроек в нужном приоритете
     * Первым делом из сессии отправленные с формы
     * Затем из БД
     * @return array
     * @throws \Exception
     */
    public function getSettingsValue()
    {
        $storeId = (int)$this->config->get('config_store_id');
        $dbSettings = $this->modelSetting->getSetting($this->prefixSettings, $storeId);
        $fields = $this->settingFields();
        $settings = [];
        foreach ($fields as $fieldName => $fieldSetting) {
            if (isset($this->session->data[$this->prefixSettings . $fieldName])) {
                $settings[$fieldName] = $this->session->data[$this->prefixSettings . $fieldName];
                unset($this->session->data[$this->prefixSettings . $fieldName]);
            } elseif (!empty($dbSettings[$this->prefixSettings . $fieldName])) {
                $settings[$fieldName] = $dbSettings[$this->prefixSettings . $fieldName];
            } else {
                $settings[$fieldName] = null;
            }
        }
        return $settings;
    }

    /**
     * Выставляет default значения настроек
     * @throws \Exception
     */
    public function setDefaultValueSettings()
    {
        $newSettings = [];
        $defaultSettings = $this->defaultValues();
        // Смотрим какие настройки уже есть в БД, чтобы не затереть их при повторной установке
        $storeId = (int)$this->config->get('config_store_id');
        $dbSettings = $this->modelSetting->getSetting($this->prefixSettings, $storeId);
        foreach ($defaultSettings as $fieldName => $fieldValue) {
            if (isset($dbSettings[$this->prefixSettings . $fieldName])) {
                $newSettings[$this->prefixSettings . $fieldName] = $dbSettings[$this->prefixSettings . $fieldName];
            } else {
                $newSettings[$this->prefixSettings . $fieldName] = $fieldValue;
            }
        }
        $this->modelSetting->editSetting($this->prefixSettings, $newSettings, $storeId);
        $this->log->log('Выставили значения настроек при установке  - ' . print_r($newSettings, true));
    }

    /**
     * Выставляет значения нсатроек из запроса в сессию, чтобы после запроса использовать
     * @param array $data данные из запроса
     */
    public function setValueInSession($data)
    {
        $fields = $this->settingFields();
        foreach ($fields as $field => $fieldSetting) {
            if (isset($data[$this->prefixSettings . $field])) {
                $this->session->data[$this->prefixSettings . $field] = $data[$this->prefixSettings . $field];
            }
        }
    }

    /**
     * Сохранение настроек
     * @param array $settings
     * @throws \Exception
     */
    public function save($settings)
    {
        $storeId = (int)$this->config->get('config_store_id');
        foreach ($settings as $settingName => $settingValue) {
            $this->modelSetting->editSettingValue(
                $this->prefixSettings,
                $this->prefixSettings . $settingName,
                $settingValue,
                $storeId
            );
        }
        $this->session->data['success'] = $this->language->get('saved');
        $this->log->log('Были обновлены настройки ' . print_r($settings, true));
    }

    /**
     * Определяет и возвращает префикс настроек в зависимости от версии CMS
     * @return string
     */
    private function determinePrefixSettings()
    {
        if ($this->version == 2) {
            return self::PREFIX_SETTINGS_VERSION_2;
        } elseif ($this->version == 3) {
            return self::PREFIX_SETTINGS_VERSION_3;
        }
        // @todo: если первая версия CMS, что нас ждет там
        return 'delo_';
    }
}
